@@ -21,44 +21,47 @@
 
 #include "config.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <string.h>
-
+#if STDC_HEADERS
+# include <stdio.h>
+# include <stdlib.h>
+# include <stdarg.h>
+#endif  /* STDC_HEADERS */
+#if HAVE_STRING_H
+# include <string.h>
+#endif  /* HAVE_STRING_H */
 #if HAVE_UNISTD_H
 # include <unistd.h>
-#endif
+#endif  /* HAVE_UNISTD_H */
 #if HAVE_SYS_UNISTD_H
 # include <sys/unistd.h>
-#endif
+#endif  /* HAVE_UNISTD_H */
 #if HAVE_SYS_TYPES_H
 #include <sys/types.h>
-#endif
+#endif  /* HAVE_SYS_TYPES_H */
 #if HAVE_SYS_SELECT_H
 #include <sys/select.h>
-#endif
+#endif  /* HAVE_SYS_SELECT_H */
 #if HAVE_TIME_H
 # include <time.h>
-#endif
+#endif  /* HAVE_TIME_H */
 #if HAVE_SYS_TIME_H
 # include <sys/time.h>
-#endif
+#endif  /* HAVE_SYS_TIME_H */
 #if HAVE_INTTYPES_H
 # include <inttypes.h>
-#endif
+#endif  /* HAVE_INTTYPES_H */
 #if HAVE_ERRNO_H
 # include <errno.h>
-#endif
+#endif  /* HAVE_ERRNO_H */
 #if HAVE_TERMIOS_H
 # include <termios.h>
-#endif
+#endif  /* HAVE_TERMIOS_H */
 #if HAVE_SYS_IOCTL_H
 # include <sys/ioctl.h>
-#endif
+#endif  /* HAVE_SYS_IOCTL_H */
 #if HAVE_IO_H
 # include <io.h>
-#endif
+#endif  /* HAVE_IO_H */
 
 #include "decoder.h"
 
@@ -315,6 +318,11 @@ sixel_decoder_decode(
         goto end;
     }
 
+    if (sx > SIXEL_WIDTH_LIMIT || sy > SIXEL_HEIGHT_LIMIT) {
+        status = SIXEL_BAD_INPUT;
+        goto end;
+    }
+
     status = sixel_helper_write_image_file(indexed_pixels, sx, sy, palette,
                                            SIXEL_PIXELFORMAT_PAL8,
                                            decoder->output,
