@@ -21,18 +21,22 @@
 
 #include "config.h"
 
-#include <string.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <math.h>
-
+#if STDC_HEADERS
+# include <stdlib.h>
+# include <stdio.h>
+#endif  /* STDC_HEADERS */
+#if HAVE_MATH_H
+# include <math.h>
+#endif  /* HAVE_MATH_H */
+#if HAVE_STRING_H
+# include <string.h>
+#endif  /* HAVE_STRING_H */
 #if HAVE_LIMITS_H
 # include <limits.h>
-#endif
-
+#endif  /* HAVE_LIMITS_H */
 #if HAVE_INTTYPES_H
 # include <inttypes.h>
-#endif
+#endif  /* HAVE_INTTYPES_H */
 
 #include "dither.h"
 #include "quant.h"
@@ -262,6 +266,7 @@ sixel_dither_new(
     size_t wholesize;
     int quality_mode;
 
+    /* ensure given pointer is not null */
     if (ppdither == NULL) {
         sixel_helper_set_additional_message(
             "sixel_dither_new: ppdither is null.");
@@ -280,13 +285,17 @@ sixel_dither_new(
     }
 
     if (ncolors < 0) {
-        ncolors = 256;
+        ncolors = SIXEL_PALETTE_MAX;
         quality_mode = SIXEL_QUALITY_HIGHCOLOR;
     } else {
         if (ncolors > SIXEL_PALETTE_MAX) {
-            ncolors = 256;
-        } else if (ncolors < 2) {
-            ncolors = 2;
+            status = SIXEL_BAD_INPUT;
+            goto end;
+        } else if (ncolors < 1) {
+            status = SIXEL_BAD_INPUT;
+            sixel_helper_set_additional_message(
+                "sixel_dither_new: palette colors must be more than 0");
+            goto end;
         }
         quality_mode = SIXEL_QUALITY_LOW;
     }
@@ -519,6 +528,7 @@ sixel_dither_initialize(
     unsigned char *input_pixels;
     SIXELSTATUS status = SIXEL_FALSE;
 
+    /* ensure dither object is not null */
     if (dither == NULL) {
         sixel_helper_set_additional_message(
             "sixel_dither_new: dither is null.");
@@ -526,6 +536,7 @@ sixel_dither_initialize(
         goto end;
     }
 
+    /* increment ref count */
     sixel_dither_ref(dither);
 
     sixel_dither_set_pixelformat(dither, pixelformat);
@@ -589,7 +600,10 @@ sixel_dither_initialize(
 
 end:
     free(normalized_pixels);
+
+    /* decrement ref count */
     sixel_dither_unref(dither);
+
     return status;
 }
 
@@ -710,7 +724,7 @@ sixel_dither_set_transparent(
 
 
 /* set transparent */
-SIXELAPI unsigned char *
+SIXELAPI sixel_index_t *
 sixel_dither_apply_palette(
     sixel_dither_t  /* in */ *dither,
     unsigned char   /* in */ *pixels,
@@ -719,11 +733,12 @@ sixel_dither_apply_palette(
 {
     SIXELSTATUS status = SIXEL_FALSE;
     size_t bufsize;
-    unsigned char *dest = NULL;
+    sixel_index_t *dest = NULL;
     int ncolors;
     unsigned char *normalized_pixels = NULL;
     unsigned char *input_pixels;
 
+    /* ensure dither object is not null */
     if (dither == NULL) {
         sixel_helper_set_additional_message(
             "sixel_dither_apply_palette: dither is null.");
@@ -733,8 +748,8 @@ sixel_dither_apply_palette(
 
     sixel_dither_ref(dither);
 
-    bufsize = (size_t)(width * height) * sizeof(unsigned char);
-    dest = (unsigned char *)sixel_allocator_malloc(dither->allocator, bufsize);
+    bufsize = (size_t)(width * height) * sizeof(sixel_index_t);
+    dest = (sixel_index_t *)sixel_allocator_malloc(dither->allocator, bufsize);
     if (dest == NULL) {
         sixel_helper_set_additional_message(
             "sixel_dither_new: sixel_allocator_malloc() failed.");
@@ -821,7 +836,7 @@ test1(void)
 #  pragma GCC diagnostic push
 #  pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 #endif
-    dither = sixel_dither_create(0);
+    dither = sixel_dither_create(2);
 #if HAVE_DIAGNOSTIC_DEPRECATED_DECLARATIONS
 #  pragma GCC diagnostic pop
 #endif
