@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2018 Hayaki Saito
+ * Copyright (c) 2014-2019 Hayaki Saito
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
  * this software and associated documentation files (the "Software"), to deal in
@@ -21,22 +21,19 @@
 
 #include "config.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
+#if STDC_HEADERS
+# include <stdio.h>
+# include <stdlib.h>
+#endif
+#if HAVE_STRING_H
+# include <string.h>
+#endif
 #if HAVE_UNISTD_H
 # include <unistd.h>
 #endif
-
-#ifdef HAVE_ERRNO_H
+#if HAVE_ERRNO_H
 # include <errno.h>
 #endif
-
-#if !defined(HAVE_MEMCPY)
-# define memcpy(d, s, n) (bcopy ((s), (d), (n)))
-#endif
-
 #ifdef HAVE_GDK_PIXBUF2
 # if HAVE_DIAGNOSTIC_TYPEDEF_REDEFINITION
 #   pragma GCC diagnostic push
@@ -47,20 +44,20 @@
 #   pragma GCC diagnostic pop
 # endif
 #endif
-
-#ifdef HAVE_GD
+#if HAVE_GD
 # include <gd.h>
 #endif
-
-#ifdef HAVE_LIBPNG
+#if HAVE_LIBPNG
 # include <png.h>
 #endif  /* HAVE_LIBPNG */
-
 #if HAVE_JPEG
-# include <stdio.h>
 # include <jpeglib.h>
 #endif  /* HAVE_JPEG */
 
+#if !defined(HAVE_MEMCPY)
+# define memcpy(d, s, n) (bcopy ((s), (d), (n)))
+#endif
+
 #include "frame.h"
 #include <sixel.h>
 #include "chunk.h"
@@ -94,6 +91,7 @@ stbi_free(void *p)
 
 #define STBI_NO_STDIO 1
 #define STB_IMAGE_IMPLEMENTATION 1
+#define STBI_FAILURE_USERMSG 1
 #define STBI_NO_GIF
 #define STBI_NO_PNM
 
@@ -121,7 +119,14 @@ stbi_free(void *p)
 # pragma GCC diagnostic push
 # pragma GCC diagnostic ignored "-Wunused-function"
 #endif
+# if HAVE_DIAGNOSTIC_UNUSED_BUT_SET_VARIABLE
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wunused-but-set-variable"
+#endif
 #include "stb_image.h"
+#if HAVE_DIAGNOSTIC_UNUSED_BUT_SET_VARIABLE
+# pragma GCC diagnostic pop
+#endif
 #if HAVE_DIAGNOSTIC_UNUSED_FUNCTION
 # pragma GCC diagnostic pop
 #endif
@@ -195,6 +200,12 @@ load_jpeg(unsigned char **result,
 
     while (cinfo.output_scanline < cinfo.output_height) {
         jpeg_read_scanlines(&cinfo, buffer, 1);
+        if (cinfo.err->num_warnings > 0) {
+            sixel_helper_set_additional_message(
+                "jpeg_read_scanlines: error/warining occuered.");
+            status = SIXEL_BAD_INPUT;
+            goto end;
+        }
         memcpy(*result + (cinfo.output_scanline - 1) * row_stride, buffer[0], row_stride);
     }
 
@@ -262,6 +273,20 @@ read_palette(png_structp png_ptr,
     }
 }
 
+jmp_buf jmpbuf;
+
+/* libpng error handler */
+static void
+png_error_callback(png_structp png_ptr, png_const_charp error_message)
+{
+    (void) png_ptr;
+
+    sixel_helper_set_additional_message(error_message);
+#if HAVE_SETJMP && HAVE_LONGJMP
+    longjmp(jmpbuf, 1);
+#endif  /* HAVE_SETJMP && HAVE_LONGJMP */
+}
+
 
 static SIXELSTATUS
 load_png(unsigned char      /* out */ **result,
@@ -277,26 +302,54 @@ load_png(unsigned char      /* out */ **result,
          int                /* out */ *transparent,
          sixel_allocator_t  /* in */  *allocator)
 {
-    SIXELSTATUS status = SIXEL_FALSE;
+    SIXELSTATUS status;
     sixel_chunk_t read_chunk;
     png_uint_32 bitdepth;
     png_uint_32 png_status;
     png_structp png_ptr;
     png_infop info_ptr;
-    unsigned char **rows = NULL;
+#ifdef HAVE_DIAGNOSTIC_CLOBBERED
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wclobbered"
+#endif
+    unsigned char **rows;
     png_color *png_palette = NULL;
     png_color_16 background;
     png_color_16p default_background;
     int i;
     int depth;
 
-    png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
+#if HAVE_SETJMP && HAVE_LONGJMP
+    if (setjmp(jmpbuf) != 0) {
+        sixel_allocator_free(allocator, *result);
+        *result = NULL;
+        status = SIXEL_PNG_ERROR;
+        goto cleanup;
+    }
+#endif  /* HAVE_SETJMP && HAVE_LONGJMP */
+
+    status = SIXEL_FALSE;
+    rows = NULL;
+    *result = NULL;
+
+    png_ptr = png_create_read_struct(
+        PNG_LIBPNG_VER_STRING, NULL, &png_error_callback, NULL);
     if (!png_ptr) {
         sixel_helper_set_additional_message(
             "png_create_read_struct() failed.");
         status = SIXEL_PNG_ERROR;
         goto cleanup;
     }
+
+#if HAVE_SETJMP
+    if (setjmp(png_jmpbuf(png_ptr)) != 0) {
+        sixel_allocator_free(allocator, *result);
+        *result = NULL;
+        status = SIXEL_PNG_ERROR;
+        goto cleanup;
+    }
+#endif  /* HAVE_SETJMP */
+
     info_ptr = png_create_info_struct(png_ptr);
     if (!info_ptr) {
         sixel_helper_set_additional_message(
@@ -570,14 +623,7 @@ load_png(unsigned char      /* out */ **result,
         }
         break;
     }
-#if USE_SETJMP && HAVE_SETJMP
-    if (setjmp(png_jmpbuf(png_ptr))) {
-        sixel_allocator_free(allocator, *result);
-        *result = NULL;
-        status = SIXEL_PNG_ERROR;
-        goto cleanup;
-    }
-#endif  /* HAVE_SETJMP */
+
     png_read_image(png_ptr, rows);
 
     status = SIXEL_OK;
@@ -588,7 +634,11 @@ cleanup:
 
     return status;
 }
-# endif  /* HAVE_PNG */
+#ifdef HAVE_DIAGNOSTIC_CLOBBERED
+# pragma GCC diagnostic pop
+#endif
+
+# endif  /* HAVE_LIBPNG */
 
 
 static SIXELSTATUS
@@ -941,10 +991,11 @@ load_with_gdkpixbuf(
     GdkPixbuf *pixbuf;
     GdkPixbufAnimation *animation;
     GdkPixbufLoader *loader = NULL;
-#if 1
     GdkPixbufAnimationIter *it;
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
     GTimeVal time_val;
-#endif
+#pragma GCC diagnostic pop
     sixel_frame_t *frame = NULL;
     int stride;
     unsigned char *p;
@@ -963,7 +1014,10 @@ load_with_gdkpixbuf(
 #if (!GLIB_CHECK_VERSION(2, 36, 0))
     g_type_init();
 #endif
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
     g_get_current_time(&time_val);
+#pragma GCC diagnostic pop
     loader = gdk_pixbuf_loader_new();
     gdk_pixbuf_loader_write(loader, pchunk->buffer, pchunk->size, NULL);
     animation = gdk_pixbuf_loader_get_animation(loader);
@@ -1006,7 +1060,10 @@ load_with_gdkpixbuf(
             goto end;
         }
     } else {
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
         g_get_current_time(&time_val);
+#pragma GCC diagnostic pop
 
         frame->frame_no = 0;
 
@@ -1014,7 +1071,10 @@ load_with_gdkpixbuf(
         for (;;) {
             while (!gdk_pixbuf_animation_iter_on_currently_loading_frame(it)) {
                 frame->delay = gdk_pixbuf_animation_iter_get_delay_time(it);
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
                 g_time_val_add(&time_val, frame->delay * 1000);
+#pragma GCC diagnostic pop
                 frame->delay /= 10;
                 pixbuf = gdk_pixbuf_animation_iter_get_pixbuf(it);
                 if (pixbuf == NULL) {
@@ -1291,21 +1351,27 @@ end:
 SIXELAPI SIXELSTATUS
 sixel_helper_load_image_file(
     char const                /* in */     *filename,     /* source file name */
-    int                       /* in */     fstatic,       /* whether to extract static image */
-    int                       /* in */     fuse_palette,  /* whether to use paletted image */
-    int                       /* in */     reqcolors,     /* requested number of colors */
-    unsigned char             /* in */     *bgcolor,      /* background color */
+    int                       /* in */     fstatic,       /* whether to extract static image from animated gif */
+    int                       /* in */     fuse_palette,  /* whether to use paletted image, set non-zero value to try to get paletted image */
+    int                       /* in */     reqcolors,     /* requested number of colors, should be equal or less than SIXEL_PALETTE_MAX */
+    unsigned char             /* in */     *bgcolor,      /* background color, may be NULL */
     int                       /* in */     loop_control,  /* one of enum loopControl */
     sixel_load_image_function /* in */     fn_load,       /* callback */
     int                       /* in */     finsecure,     /* true if do not verify SSL */
-    int const                 /* in */     *cancel_flag,  /* cancel flag */
-    void                      /* in/out */ *context,      /* private data */
-    sixel_allocator_t         /* in */     *allocator     /* allocator object */
+    int const                 /* in */     *cancel_flag,  /* cancel flag, may be NULL */
+    void                      /* in/out */ *context,      /* private data which is passed to callback function as an argument, may be NULL */
+    sixel_allocator_t         /* in */     *allocator     /* allocator object, may be NULL */
 )
 {
     SIXELSTATUS status = SIXEL_FALSE;
     sixel_chunk_t *pchunk = NULL;
 
+    /* normalize reqested colors */
+    if (reqcolors > SIXEL_PALETTE_MAX) {
+        reqcolors = SIXEL_PALETTE_MAX;
+    }
+
+    /* create new chunk object from file */
     status = sixel_chunk_new(&pchunk, filename, finsecure, cancel_flag, allocator);
     if (status != SIXEL_OK) {
         goto end;
@@ -1377,7 +1443,7 @@ test1(void)
     unsigned char *ptr = malloc(16);
 
     nret = EXIT_SUCCESS;
-    goto error; 
+    goto error;
 
     nret = EXIT_SUCCESS;
 
