@@ -46,18 +46,22 @@
 
 #include "config.h"
 
-#include <string.h>
-#include <stdlib.h>
-#include <stdio.h>
+#if STDC_HEADERS
+# include <stdlib.h>
+# include <stdio.h>
+#endif  /* STDC_HEADERS */
+#if HAVE_STRING_H
+# include <string.h>
+#endif  /* HAVE_STRING_H */
+#if HAVE_MATH_H
 #include <math.h>
-
+#endif  /* HAVE_MATH_H */
 #if HAVE_LIMITS_H
 # include <limits.h>
-#endif
-
+#endif  /* HAVE_MATH_H */
 #if HAVE_INTTYPES_H
 # include <inttypes.h>
-#endif
+#endif  /* HAVE_MATH_H */
 
 #include "quant.h"
 
@@ -138,7 +142,8 @@ alloctupletable(
     sixel_allocator_t   /* in */  *allocator)
 {
     SIXELSTATUS status = SIXEL_FALSE;
-    char message[256];
+    enum { message_buffer_size = 256 };
+    char message[message_buffer_size];
     int nwrite;
     unsigned int mainTableSize;
     unsigned int tupleIntSize;
@@ -748,6 +753,7 @@ computeHistogram(unsigned char const    /* in */  *data,
     }
 
     colorfreqtableP->size = (unsigned int)(ref - refmap);
+
     status = alloctupletable(&colorfreqtableP->table, depth, (unsigned int)(ref - refmap), allocator);
     if (SIXEL_FAILED(status)) {
         goto end;
@@ -1240,7 +1246,7 @@ end:
 /* apply color palette into specified pixel buffers */
 SIXELSTATUS
 sixel_quant_apply_palette(
-    unsigned char     /* out */ *result,
+    sixel_index_t     /* out */ *result,
     unsigned char     /* in */  *data,
     int               /* in */  width,
     int               /* in */  height,
@@ -1262,8 +1268,8 @@ sixel_quant_apply_palette(
     component_t offset;
     int color_index;
     unsigned short *indextable;
-    unsigned char new_palette[256 * max_depth];
-    unsigned short migration_map[256];
+    unsigned char new_palette[SIXEL_PALETTE_MAX * 4];
+    unsigned short migration_map[SIXEL_PALETTE_MAX];
     float (*f_mask) (int x, int y, int c) = NULL;
     void (*f_diffuse)(unsigned char *data, int width, int height,
                       int x, int y, int depth, int offset);
@@ -1360,7 +1366,7 @@ sixel_quant_apply_palette(
     if (foptimize_palette) {
         *ncolors = 0;
 
-        memset(new_palette, 0x00, sizeof(256 * depth));
+        memset(new_palette, 0x00, sizeof(SIXEL_PALETTE_MAX * depth));
         memset(migration_map, 0x00, sizeof(migration_map));
 
         if (f_mask) {
