@@ -21,23 +21,24 @@
 
 #include "config.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
+#if STDC_HEADERS
+# include <stdio.h>
+# include <stdlib.h>
+#endif  /* STDC_HEADERS */
+#if HAVE_STRING_H
+# include <string.h>
+#endif  /* HAVE_STRING_H */
 #if HAVE_SETJMP_H
 # include <setjmp.h>
-#endif
-
+#endif  /* HAVE_SETJMP_H */
 #if HAVE_ERRNO_H
 # include <errno.h>
-#endif
-
+#endif  /* HAVE_ERRNO_H */
 #if HAVE_LIBPNG
 # include <png.h>
 #else
 # include "stb_image_write.h"
-#endif
+#endif  /* HAVE_LIBPNG */
 
 #include <sixel.h>
 
@@ -317,6 +318,46 @@ sixel_helper_write_image_file(
         sixel_allocator_ref(allocator);
     }
 
+    if (width > SIXEL_WIDTH_LIMIT) {
+        sixel_helper_set_additional_message(
+            "sixel_encode: bad width parameter."
+            " (width > SIXEL_WIDTH_LIMIT)");
+        status = SIXEL_BAD_INPUT;
+        goto end;
+    }
+
+    if (width > SIXEL_HEIGHT_LIMIT) {
+        sixel_helper_set_additional_message(
+            "sixel_encode: bad width parameter."
+            " (width > SIXEL_HEIGHT_LIMIT)");
+        status = SIXEL_BAD_INPUT;
+        goto end;
+    }
+
+    if (height < 1) {
+        sixel_helper_set_additional_message(
+            "sixel_encode: bad height parameter."
+            " (height < 1)");
+        status = SIXEL_BAD_INPUT;
+        goto end;
+    }
+
+    if (width < 1) {
+        sixel_helper_set_additional_message(
+            "sixel_encode: bad width parameter."
+            " (width < 1)");
+        status = SIXEL_BAD_INPUT;
+        goto end;
+    }
+
+    if (height < 1) {
+        sixel_helper_set_additional_message(
+            "sixel_encode: bad height parameter."
+            " (height < 1)");
+        status = SIXEL_BAD_INPUT;
+        goto end;
+    }
+
     switch (imageformat) {
     case SIXEL_FORMAT_PNG:
         status = write_png_to_file(data, width, height, palette,
